import { useEffect } from "react";
import { useLastRound } from "../../../domain/gameStore";
import useMap from "../../../hooks/useMap";
import usePlayerScores from "../../../hooks/usePlayerScores";
import usePlayerColors from "./usePlayerColors";
import { makeQuestionMarker, makeFlagMarker, makeLine } from "./markers";

const useMapWithMarkers = (mapDivRef) => {
  // get the info about the last round
  const { roundNum, targetPoint } = useLastRound();

  // create the map
  const mapRef = useMap(mapDivRef, targetPoint.lat, targetPoint.lng, 4);

  // set up the flag at the target point
  useEffect(() => {
    const targetMarker = makeFlagMarker(mapRef.current, targetPoint);
    return () => targetMarker.setMap(null);
  }, [mapRef, targetPoint]);

  // live update with all the scores
  const players = usePlayerScores()
    ?.filter(({ guesses }) => guesses[roundNum] && guesses[roundNum].score !== null)
    ?.map(({ name, totalScore, guesses }) => ({ name, totalScore, guess: guesses[roundNum] }));

  // set up the marker colors
  const playerColors = usePlayerColors(players?.map(({ name }) => name));

  // draw all the player markers and trails
  useEffect(() => {
    if (!players) return;
    const drawings = [];
    players.forEach(({ name, guess: { lat, lng, score } }) => {
      const color = playerColors[name];
      const selectedPoint = { lat, lng };

      const marker = makeQuestionMarker(mapRef.current, selectedPoint, `${name} - ${score} Points`, color);
      drawings.push(marker);

      const line = makeLine(selectedPoint, targetPoint, mapRef.current, color);
      drawings.push(line);
    });
    return () => drawings.forEach((drawing) => drawing.setMap(null));
  }, [players, mapRef, targetPoint, playerColors]);
};

export default useMapWithMarkers;